<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 * This is the main engine.
 */
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Engine for PHPDevShell, this is what makes the system run, it calls all needed classes on each request. //
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

define('phpdevshell_version', 'PHPDevShell V 3.0.2-Stable-DB-3001');
define('phpdevshell_db_version', '3001');


require_once 'PHPDS_utils.inc.php';

/**
 * PHPDS: the main class for the PHPDevShell engine
 *
 * It can be used either standalone or embedded
 *
 * NOTE: none of these methods are meant to be called by you. For standalone mode, consult PHPDevShell programmers' guide. For embedded mode, use the PHPDSlib
 *
 * @date 20090427
 * @version	1.0
 *
 */
class PHPDS
{
	/**
	 * Core system configuration settings.
	 *
	 * @var object
	 */
	protected $configuration;
	/**
	 * Core system language values.
	 *
	 * @var object
	 */
	protected $lang;
	/**
	 * Core database object.
	 *
	 * @var object
	 */
	protected $db;
	/**
	 * Core object.
	 *
	 * @var object
	 */
	protected $core;
	/**
	 * Core user object.
	 *
	 * @var object
	 */
	protected $user;
	/**
	 * Core security object.
	 *
	 * @var object
	 */
	protected $security;
	/**
	 * Core navigation object.
	 *
	 * @var object
	 */
	protected $navigation;
	/**
	 * Core template object.
	 *
	 * @var object
	 */
	protected $template;
	/**
	 * Main instance of the debug module, which handles startup init.
	 *
	 * @var object
	 */
	protected $debugInstance;
	/**
	 * Main instance of the tagger module
	 */
	protected $tagger;
	/**
	 * PHPDS is used throught the lib (true) or standalone (false).
	 *
	 * @var boolean
	 */
	protected $embedded;
	/**
	 * The textual path of PHPDS directory on disk (not the URL).
	 *
	 * @var string
	 */
	protected $basepath;
	/**
	 * The higher the less backward-compatible.
	 *
	 * @var int
	 */
	protected $compatMode = 1;
	/**
	 * Contains array of installed supportive plugin classes.
	 *
	 * @var array
	 */
	public $PluginClasses;
	/**
	 * Execution stage (i.e. run level)
	 *
	 * @var int
	 */
	protected $stage = 1; // 1 => initialization ; 2 => running
	/**
	 * Constructor: initialize the instance and starts the configuration process
	 *
	 * @param $embedded
	 * @date 20100618 (v1.0.2) (greg) added exception handler to display config parameters to allow inspection in a problematic installation
	 * @date 20091006 Improved windoz compat by using DIRECTORY_SEPARATOR
	 * @author greg
	 * @version 1.0.2
	 * @return itself
	 */
	public function __construct($embedded = false)
	{
		// Start Ob.
		ob_start();
		$this->embedded = $embedded;
		$this->basepath = realpath(dirname(__FILE__) . DIRECTORY_SEPARATOR . '..') . DIRECTORY_SEPARATOR;
		try {
			$this->config();
		} catch (Exception $e) {
			$this->PHPDS_errorHandler()->doHandleException($e);
		}
		return $this;
	}

	/**
	 * Destruct Class.
	 *
	 * @date 20091030 mute possible error with @ (in case destruct is called before ob is started)
	 * @author	greg
	 * @version 1.0.1
	 * @return nothing
	 *
	 */
	public function __destruct()
	{
		// Flush.
		PU_cleanBuffers(true);
	}

	/**
	 * Load one config file, if it exists
	 *
	 * If it's ok, its name is added to $configuration['config_files']
	 *
	 * @param $path absolute file name
	 * @return boolean	wether it's successfull
	 * @date 20090521	Added
	 * @date 20091006 Changed to dual list of files (used/missing)
	 * @date 20100630 (v1.1.1) (greg) fixed the misplaced _log() call
	 * @date 20110225 (v1.1.2) (greg) fixed _log() to log() ;  remove pre/postfix on $path
	 * @author	greg <greg@phpdevshell.org>
	 * @version 1.1.2
	 */
	protected function includeConfigFile($path, &$configuration)
	{
		if (empty($path)) return false;

		if (!file_exists($path)) {
			$configuration['config_files_missing'][] = $path;
			return false;
		}
		$this->log("Loading config file $path");
		include_once $path;
		$configuration['config_files_used'][] = $path;
		return true;
	}

	/**
	 * Load one config file, if it exists; also tried a ".local" with the same name
	 *
	 * @param $filename	short file name
	 * @return nothing
	 * @date 20090521	Added
	 * @date 20091006 Changed to dual list of files (used/missing)
	 * @version 1.0
	 */
	protected function loadConfigFile($filename, &$configuration)
	{
		if (empty($filename)) return false;

		$this->log("Looking for general config file \"$filename\"");

		$this->includeConfigFile($this->basepath('config') . $filename . '.config.php', $configuration);
	}

	/**
	 * Load plugin-specific host-style config
	 *
	 * This allows plugins to provide a configuration, for example when 1 plugin <=> 1 site
	 *
	 */
	protected function loadPluginsConfig(&$configuration)
	{
		$files = glob($this->basepath('plugins').'/*/config/host.config.php');
		foreach($files as $filename) {
			$this->log("Looking for plugin config file \"$filename\"");
			$this->includeConfigFile($filename, $configuration);
		}
	}

	/**
	 *  Create a config from the config files, and store it in the instance field
	 *
	 *  The actual configuration is loaded from two files (one is generic, the other is specific)
	 *  from the config/ folder.
	 *
	 * @date 20090427
	 * @date 20090521	Make it lighter by using loadConfigFile()
	 * @date 20091006 Changed to dual list of files (used/missing)
	 * @date 20110225 (v1.1.1) (greg) added support for plugin-specific host-style config files
	 * @version	1.1.1
	 *
	 * @return PHPDS the current instance
	 */
	protected function loadConfig()
	{
		// starts with an empty array which will contain the actual names of the configurations files used
		$configuration = array();
		$configuration['config_files_used'] = array();
		$configuration['config_files_missing'] = array();

		$configuration['phpdevshell_version'] = phpdevshell_version;
		$configuration['phpdevshell_db_version'] = phpdevshell_db_version;

		if (!is_dir($this->basepath('config'))) {
			print 'The folder "<tt>' . $this->basepath . 'config</tt>" is missing.<br />';
			print 'It must be present and contain at least one config file.<br />';
			print 'Read the install instruction on creating a config file and installing the database.<br>';
			print '<a href="http://wiki.phpdevshell.org/wiki/Installing_PHPDevShell" target="_blank">Click here to read install instructions.</a><br>';
			print 'PHPDevShell basepath is "<tt>' . $this->basepath . '</tt>".<br>';
			exit();
		}

		$this->loadConfigFile('PHPDS-defaults', $configuration);
		$this->loadConfigFile('multi-host', $configuration);
		$this->loadConfigFile('single-site', $configuration);

		$this->loadPluginsConfig($configuration);

		if (!empty($_SERVER['SERVER_NAME'])) {
			if (!empty($configuration['host'][$_SERVER['SERVER_NAME']])) {
				$this->loadConfigFile($configuration['host'][$_SERVER['SERVER_NAME']], $configuration);
			}
			$this->loadConfigFile($_SERVER['SERVER_NAME'], $configuration);
		}

		if (count($configuration['config_files_used']) == 0) {
			print 'No config file found for host "' . $_SERVER['SERVER_NAME'] . '" inside folder "<tt>' . $this->basepath('config') . '</tt>"<br>';
			print 'At least one file among those listed below should be readable.<br />';
			print 'Read the install instruction on creating a config file and installing the database.<br>';
			print '<a href="http://wiki.phpdevshell.org/wiki/Installing_PHPDevShell" target="_blank">Click here to read install instructions.</a><br>';
			print 'PHPDevShell basepath is "<tt>' . $this->basepath . '</tt>".<br>';
			print 'All these files were tried:<br><tt>';
			print implode('<br>', $configuration['config_files_missing']);
			print '</tt>';
			exit();
		}
		$this->PHPDS_configuration($configuration);
		$success = spl_autoload_register(array($this, "PHPDS_autoloader"));
		return $this; // to allow fluent interface
	}

	/**
	 * Deal with the "session" part of the configuration
	 *
	 * In standalone mode, create the sessions ; in embedded mode,
	 * fetch the current session (we hijack the session, we don't create a new one)
	 *
	 * NOTE: in embedded mode you MUST create a session before using PHPDSlib
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @return PHPDS the current instance
	 */
	protected function configSession()
	{
		if (empty($this->configuration['absolute_url'])) {
			$protocol = empty($_SERVER['HTTPS']) ? 'http://' : 'https://';
			$this->configuration['absolute_url'] = $protocol . $_SERVER['HTTP_HOST'] . str_replace('/index.php', '', $_SERVER['PHP_SELF']);
		}

		if ($this->embedded) {
			$this->configuration['session_name'] = session_name();
			// TODO: deal with empty session
		} else {
			if (!empty($this->configuration['session_life']))
					ini_set('session.gc_maxlifetime', $this->configuration['session_life']);
			if (!empty($this->configuration['absolute_url']))
					$this->configuration['session_name'] = md5($this->configuration['absolute_url']);

			if (!empty($this->configuration['session_name']))
					session_name(md5($this->configuration['session_name']));
			if (!empty($this->configuration['session_path']))
					session_save_path($this->configuration['session_path']);

			try {
				$p = realpath(session_save_path());
				session_start();
			} catch (Exception $e) {
				throw new Exception('Unable to start the session. This is probably because the session folder is not writable or missing.<br />Please check that the folder "' . session_save_path() . '" is present and writable, then reload this page.');
			}
		}
		return $this; // to allow fluent interface
	}

	/**
	 * Deal with database access configuration
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @return PHPDS the current instance
	 */
	protected function configDb()
	{
		$db = $this->PHPDS_db();
		$configuration = $this->configuration;
		$db->server = $configuration['server_address'];
		$db->dbUsername = $configuration['database_user_name'];
		$db->dbName = $configuration['database_name'];
		$db->dbPassword = $configuration['database_password'];

		if ($configuration['cache_type'] == 'memcache') $db->connectMemcache();
		return $this; // to allow fluent interface
	}

	/**
	 * Copy settings from the database-loaded array. Converts and defaults to false if the value isn't set
	 *
	 * @date 		20100219
	 * @author		greg
	 * @version		1.1
	 * 	@param		$source		array, the array to extract values from
	 * @param		$target		array, the array to add the values to
	 * @param		$indexes	array, the indexes of the values to copy
	 * @param		$type			string, the type of value to cast (currently only boolean or null for everything else)
	 * @return nothing
	 */
	protected function copySettings($settings, $type = null)
	{
		$this->copyArray($this->PHPDS_db()->essentialSettings, $this->configuration, $settings, $type);
	}

	/**
	 * Copy an array to another  and defaults to false if the value isn't set
	 *
	 * @date 			20100219
	 * @author		greg
	 * @version		1.0
	 * @param		$source		array, the array to extract values from
	 * @param		$target		array, the array to add the values to
	 * @param		$indexes	array, the indexes of the values to copy
	 * @param		$type			string, the type of value to cast (currently only boolean or null for everything else)
	 * @return 		nothing
	 */
	public function copyArray($source, &$target, $indexes, $type = null)
	{
		if (!is_array($indexes)) $indexes = array($indexes);
		switch ($type) {
			case 'boolean':
				foreach ($indexes as $index) {
					$target[$index] = isset($source[$index]) ? (boolean) $source[$index] : false;
				}
				break;
			default:
				foreach ($indexes as $index) {
					$target[$index] = isset($source[$index]) ? $source[$index] : false;
				}
		}
	}

	/**
	 * Fetch core settings from the site configuration stored in the database
	 * Also fetch some settings from the session and the locales
	 *
	 * @date 20090513
	 * @version	1.0.1
	 *
	 * @return PHPDS the current instance
	 */
	protected function configCoreSettings()
	{
		// Set core settings. ////////////////////////////////////
		$this->configuration['absolute_path'] = $this->basepath();
		$this->copySettings(array('crypt_key', 'loginandout', 'front_page_id', 'front_page_id_in', 'date_format', 'date_format_short', 'scripts_name_version', 'footer_notes', 'charset_format', 'locale_format', 'charset', 'language', 'debug_language', 'region', 'system_timezone', 'default_template', 'default_template_id', 'printable_template', 'root_id', 'root_role', 'root_group', 'setting_admin_email', 'custom_logo', 'url_append', 'skin', 'meta_keywords', 'meta_description'));
		$this->copySettings(array('allow_remember', 'system_down', 'demo_mode', 'force_core_changes', 'system_logging', 'access_logging', 'email_critical', 'sef_url', 'queries_count'), 'boolean');
		// Prepare user. /////////////////////////////////////////
		$this->PHPDS_user();
		if (!empty($this->configuration['allow_remember']) && empty($_SESSION['user_id']) && isset($_COOKIE['pdspc'])) {
			// Menu id is an integer.
			$this->PHPDS_security()->lookupPersistentUser($_COOKIE['pdspc']);
		}
		// Set crucial user settings. //////////////////////////////////////////////////////
		$this->writeUserConfig();
		////////////////////////////////////////////////////////////////////////////////////
		// Asign locale to use /////////////////////////////////////////////////////////////
		$this->configuration['locale'] = $this->configuration['user_locale']; //////////////
		// Set environment. ////////////////////////////////////////////////////////////////
		putenv('LANG=' . $this->configuration['locale']); //////////////////////////////////
		// Set locale. /////////////////////////////////////////////////////////////////////
		setlocale(LC_ALL, $this->configuration['locale']); /////////////////////////////////
		// Server date and timezones. //////////////////////////////////////////////////////
		date_default_timezone_set($this->configuration['system_timezone']); ////////////////
		// Assign clean locale directory. //////////////////////////////////////////////////
		$this->configuration['locale_dir'] = $this->PHPDS_core()->formatLocale(false); ///////
		////////////////////////////////////////////////////////////////////////////////////
		return $this; // to allow fluent interface
	}

	/**
	 * Simply writes user session data.
	 *
	 * @date 20100706
	 * @version 1.0
	 * @author Jason Schoeman
	 */
	public function writeUserConfig()
	{
		$this->configuration['user_id'] = empty($_SESSION['user_id']) ? 0 : $_SESSION['user_id'];
		$this->configuration['user_name'] = empty($_SESSION['user_name']) ? '' : $_SESSION['user_name'];
		$this->configuration['user_display_name'] = empty($_SESSION['user_display_name']) ? '' : $_SESSION['user_display_name'];
		$this->configuration['user_group'] = empty($_SESSION['user_group']) ? 0 : $_SESSION['user_group'];
		$this->configuration['user_role'] = empty($_SESSION['user_role']) ? 0 : $_SESSION['user_role'];
		$this->configuration['user_email'] = empty($_SESSION['user_email']) ? '' : $_SESSION['user_email'];
		$this->configuration['user_language'] = empty($_SESSION['user_language']) ? '' : $_SESSION['user_language'];
		$this->configuration['user_region'] = empty($_SESSION['user_region']) ? '' : $_SESSION['user_region'];
		$this->configuration['user_timezone'] = empty($_SESSION['user_timezone']) ? '' : $_SESSION['user_timezone'];
		$this->configuration['user_locale'] = empty($_SESSION['user_locale']) ? $this->PHPDS_core()->formatLocale() : $_SESSION['user_locale'];
	}

	/**
	 * Loads extra broad based system functions.
	 * @return void
	 */
	public function loadFunctions()
	{
		if (!empty($this->configuration['function_files'])) {
			foreach ($this->configuration['function_files'] as $function_file) {
				$function_file = $this->basepath('includes') . $function_file;
				if (file_exists($function_file)) {
					//$this->_log('Loading function file "'.$function_file.'"');
					include_once ($function_file);
				} //else $this->_log('Function file "'.$function_file.'" does not exist');
			}
		}
	}

	/**
	 * Main configuration method: load settings, create menus, templates, and so on
	 * After that, everything is ready to run
	 * all config_*() methods are meant to be called only at startup time by config()
	 *
	 *  @version 1.0.2
	 *
	 *  @date 20100630 (v1.0.1) (greg) moved loading of user functions up TODO: maybe we should hardcode the ones we're using??
	 *  @date 20100810 (v1.0.2) (greg) rearranged calls to allow switching to install.php if needed
	 *
	 * @return PHPDS the current instance
	 */
	protected function config()
	{
		// load various configuration files
		$this->loadConfig();

		// Does the user want utils to load.
		$this->loadFunctions();

		// init error engine.
		$this->PHPDS_errorHandler();

		// init main debug instance
		$this->PHPDS_debug();

		// various init subroutines
		$this->configSession()->configDb();

		$this->db->getEssentialSettings();

		$this->configCoreSettings();

		$this->db->installedPlugins();

		// load core language
		$this->PHPDS_core()->loadMenuLanguage();

		// This is both for templates and security: it builds the list of resources the current user is allowed to
		// Parse the request string and set the menu item.
		$this->PHPDS_navigation()->extractMenu()->parseRequestString();

		///////////////////////////////////////////////////////////////////////
		// Template Folder ////////////////////////////////////////////////////
		$this->configuration['template_folder'] = $this->PHPDS_template()->getTemplate();
		///////////////////////////////////////////////////////////////////////
		// Set core language files ////////////////////////////////////////////
		$this->PHPDS_core()->loadCoreLanguage(); ///////////////////////////////
		///////////////////////////////////////////////////////////////////////
		// Set guest if not defined ///////////////////////////////////////////
		$this->PHPDS_security()->setGuest(); ////////////////////////////////////
		///////////////////////////////////////////////////////////////////////
		// Set user session discription settings //////////////////////////////
		$this->configuration['user_display_name'] = $_SESSION['user_display_name'];
		$this->configuration['user_role_name'] = $_SESSION['user_role_name'];
		$this->configuration['user_group_name'] = $_SESSION['user_group_name'];
		///////////////////////////////////////////////////////////////////////
		// Set default plugin language files //////////////////////////////////
		$this->PHPDS_core()->loadDefaultPluginLanguage(); /////////////////////////
		///////////////////////////////////////////////////////////////////////
		$this->PluginClasses = $this->PHPDS_core()->LoadPluginClasses();
	}

	/**
	 * Actual starting point of the (non-embedded) PHPDS engine
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @return nothing
	 */
	public function run()
	{
		$this->stage = 2;
		try {
			//TODO: check we don't run an embebbed instance
			// We require templating system for final output
			// Run template as required.
			$this->PHPDS_template()->runTemplate();
			// Lets log access to menu item if so required.
			$this->PHPDS_db()->logMenuAccess();
			// Write collected logs to database.
			$this->PHPDS_db()->logThis();
			// Print queries used, when required.
			($this->configuration['queries_count']) ? $this->template->queriesUsed($this->db->countQueries) : null;
		} catch (Exception $e) {
			$this->PHPDS_errorHandler()->doHandleException($e);
		}
	}
	// accessors

	/**
	 * Create a new instance of the given class and link it as dependant (arguments as an array)
	 *
	 * @param PHPDS|PHPDS_dependant $dependancy
	 * @param  string $classname name of the class to instanciate
	 * @param array $params parameters to feed the object's constructor
	 *
	 * @date 20100217
	 * @date 20100426 (v1.0.1) (greg) direct owner is now sent instead of the top owner
	 * @date 20100922 (v1.1) (greg) moved from PHPDS_dependant to PHDS
	 *
	 * @version 1.1
	 * @author greg
	 *
	 * @return instance of $classname
	 */
	public function _factory($classname, $params = null, $dependancy = null)
	{
		if (!is_array($params)) $params = array();
		if (empty($dependancy)) $dependancy = $this;

		try {
			if (class_exists($classname, true)) {
				array_push($params, $this);
				$string = '';
				$count = count($params);
				for ($loop = 0; $loop < $count; $loop++) {
					if ($string) $string .= ', ';
					$string .= '$params[' . $loop . ']';
				}
				$c = "\$o = new $classname($string);";
				eval($c);
				if ($o instanceof $classname) {
					$o->PHPDS_dependance($dependancy);
					return $o;
				}
				throw new PHPDS_exception("Unable to factor a new \"$classname\".");
			} else {
				throw new PHPDS_exception("Can't find definition for class \"$classname\".");
			}
		} catch (Exception $e) {
			throw new PHPDS_exception("Error while factoring a new \"$classname\".", 0, $e);
		}
	}
	/*
	 * Most fields contains objects which are "lazily created" (that is created
	 * the first time they are asked for).
	 */

	/**
	 * Allow access to configuration, either read (no param) or write
	 *
	 * This makes possible to start with a forced configuration, for testing for example
	 *
	 * It returns the configuration array
	 *
	 * CAUTION: an array is not an object so be carefull to use & if you need to modify it
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @param $configuration possibly a new configuration array
	 * @return array the configuration array
	 */
	public function PHPDS_configuration($configuration = null)
	{
		if (empty($this->configuration) && is_array($configuration)) {
			$this->configuration = new PHPDS_array($configuration);
			$this->configuration['time'] = time();
			if ($this->compatMode < 2)
					$GLOBALS['configuration'] = & $this->configuration;
		}
		return $this->configuration;
	}

	/**
	 * Custom Error Handler.
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own core subsystem
	 *
	 * @date 20090618
	 * @version	1.0
	 *
	 * @return error
	 */
	public function PHPDS_errorHandler()
	{
		if (empty($this->errorHandler)) {
			$this->errorHandler = $this->_factory('PHPDS_errorHandler');
			if ($this->compatMode < 2) $GLOBALS['errorHandler'] = & $this->errorHandler;
		}
		return $this->errorHandler;
	}

	/**
	 * Allow access to the (formerly) global core subsystem
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own core subsystem
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @return core
	 */
	public function PHPDS_core()
	{
		if (empty($this->core)) {
			$this->core = $this->_factory('PHPDS_core');
			if ($this->compatMode < 2) $GLOBALS['core'] = & $this->core;
		}
		return $this->core;
	}

	/**
	 * Provides a variaty of user functions.
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own core subsystem
	 *
	 * @date 20101111
	 * @version	1.0
	 *
	 * @return user
	 */
	public function PHPDS_user()
	{
		if (empty($this->user)) {
			$this->user = $this->_factory('PHPDS_user');
			if ($this->compatMode < 2) $GLOBALS['user'] = & $this->user;
		}
		return $this->user;
	}

	/**
	 * Allow access to the global debugging subsystem
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own core subsystem
	 *
	 * @date 20100412	(v1.0.1) default domain is now "skel"
	 * @date 20090615 (v1.0) creation
	 * @version	1.0.1
	 *
	 *
	 * @return debug instance
	 */
	public function PHPDS_debug()
	{
		if (empty($this->debug)) {
			$domain = ($this->embedded ? 'authlib' : 'skel');
			$this->debug = $this->_factory('PHPDS_debug', $domain);
		}
		return $this->debug;
	}

	/**
	 * Send info data to the debug subsystem (console, firebug, ...)
	 *
	 * The goal of this function is to be called all thourough the code to be able to track bugs.
	 *
	 * @param string $data
	 */
	public function log($data)
	{
		if (!empty($this->debug)) $this->PHPDS_debug()->debug($data);
		else {
			if (!empty($GLOBALS['early_debug'])) error_log('EARLY INFO: ' . $data);
		}
	}

	/**
	 * Alias for log() - don't use it doesn't follow the guidelines
	 * @deprecated
	 * @param unknown_type $data
	 */
	public function _log($data)
	{
		return $this->log($data);
	}

	/**
	 * Allow access to the (formerly) global navigation subsystem
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own navigation subsystem
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @return navigation
	 */
	public function PHPDS_navigation()
	{
		if (empty($this->navigation)) {
			$this->navigation = $this->_factory('PHPDS_navigation');
			if ($this->compatMode < 2) $GLOBALS['navigation'] = & $this->navigation;
		}
		return $this->navigation;
	}

	/**
	 * Allow access to the (formerly) global database subsystem
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own database subsystem
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @return db
	 */
	public function PHPDS_db()
	{
		if (empty($this->db)) {
			$this->db = $this->_factory('PHPDS_db');
			if ($this->compatMode < 2) $GLOBALS['db'] = & $this->db;
		}
		return $this->db;
	}

	/**
	 * Allow access to the (formerly) global security subsystem
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own security subsystem
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @return security
	 */
	public function PHPDS_security()
	{
		if (empty($this->security)) {
			$this->security = $this->_factory('PHPDS_security');
			if ($this->compatMode < 2) $GLOBALS['security'] = & $this->security;
		}
		return $this->security;
	}

	/**
	 * Allow access to the (formerly) global templating subsystem
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own templating subsystem
	 *
	 * @date 20100928 (v1.1) (greg) added $lazy param
	 * @date 20090427 (v1.0) (greg) created
	 * @version	1.1
	 * @param boolean $lazy if true (default) the template is created if wasn't before
	 *
	 * @return template
	 */
	public function PHPDS_template($lazy = true)
	{
		if (empty($this->template) && $lazy) {
			$this->template = $this->_factory('PHPDS_template');
			if ($this->compatMode < 2) $GLOBALS['template'] = & $this->template;
		}
		return $this->template;
	}

	/**
	 * Allow access to the tagging subsystem
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own tagging subsystem
	 *
	 * @date 20100825
	 * @version	1.0
	 * @author greg
	 *
	 * @return PHPDS_tagger
	 */
	public function PHPDS_tagger()
	{
		if (empty($this->tagger)) {
			$this->tagger = $this->_factory(('PHPDS_tagger'));
		}
		return $this->tagger;
	}

	/**
	 * Allow access to the (formerly) global templating subsystem
	 *
	 * One is created if necessary.
	 *
	 * You can override to use you own templating subsystem
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @return template
	 */
	public function PHPDS_lang()
	{
		if (empty($this->lang)) {
			$this->lang = new PHPDS_array();
			if ($this->compatMode < 2) $GLOBALS['lang'] = & $this->lang;
		}
		return $this->lang;
	}

	/**
	 * This is a generic accessor to allow field access through an homogeneous and controlled way.
	 * For example:
	 *
	 * $instance->get('core')
	 *
	 * will return the core object as returned by the core() accessor method
	 *
	 * @date 20090427
	 * @version	1.0
	 *
	 * @param $field
	 * @return mixed depends on what is asked for
	 */
	public function get($field)
	{
		$method_name = 'PHPDS_' . $field;
		if (method_exists($this, $method_name)) {
			return $this->$method_name();
		}
		throw new Exception('Class "' . get_class($this) . '" has no field "' . $field . '", sorry!');
	}

	/**
	 * Check if instance is embedded.
	 *
	 * @return mixed
	 */
	public function isEmbedded()
	{
		return !empty($this->embedded);
	}

	/**
	 * Simply returns basepath
	 *
	 * An optional postfix (i.e. folder name) can be given to retrieve the path a subfolder
	 *
	 * @author greg
	 * @date 2009106 fix a problem when the path is invalid
	 * @version	1.0.1
	 * @return string
	 */
	public function basepath($postfix = '')
	{
		$path = realpath($this->basepath . $postfix);
		if ($path) return $path . DIRECTORY_SEPARATOR; else return false;
	}

	/**
	 * Try to load a class from a file
	 *
	 * NOTE: for performance reason we DON'T try first to see if the class already exists
	 *
	 * @deprecated
	 * @param $classname name of the class to look for
	 * @param $filename name of the file to look into
	 * @return boolean wether we found the class or not
	 */
	public function sneak_class($classname, $filename)
	{
		$this->sneakClass($classname, $filename);
	}

	/**
	 * Try to load a class from a file
	 *
	 * NOTE: for performance reason we DON'T try first to see if the class already exists
	 *
	 * @param $classname name of the class to look for
	 * @param $filename name of the file to look into
	 * @return boolean wether we found the class or not
	 */
	public function sneakClass($classname, $filename)
	{
		if (is_file($filename)) {
			include_once ($filename);
			if (class_exists($classname, false)) {
				$this->_log("Autoloading $classname from $filename");
				return true;
			}
		}
		return false;
	}

	/**
	 * Autoloader: when a class is instanciated, this method will load the proper php file
	 *
	 * Note: the various folders where the files are looked for depends on the
	 * 	instance configuration, and on the current plugin
	 *
	 * A model file is also loaded if present
	 *
	 * @date 20100215 (greg) added manual overloading by configuration file ; enable cascading in case a file is found but the doesn't define the class
	 * @version 1.2
	 * @author jason
	 * @param $class_name
	 * @return irrelevant, it's a callback for the new() operator (however we return true on success, false otherwise)
	 */
	function PHPDS_autoloader($class_name)
	{
		$configuration = $this->PHPDS_configuration();
		$absolute_path = $this->basepath();

		// Check if we have plugin classes to load.
		if (!empty($this->PluginClasses[$class_name])) {
			$engine_include_path = $absolute_path . 'plugins/' . $this->PluginClasses[$class_name]['plugin_folder'] . '/includes/' . $class_name . '.class.php';
			$query_include_path = $absolute_path . 'plugins/' . $this->PluginClasses[$class_name]['plugin_folder'] . '/models/' . $class_name . '.query.php';
			if ($this->sneakClass($class_name, $engine_include_path)) {
				$this->sneakClass($class_name, $query_include_path);
				return true;
			}
		}

		// Engine classes default directories
		$includes = array('includes/local', 'includes', 'includes/legacy');
		foreach ($includes as $path) {
			$engine_include_path = $absolute_path . $path . '/' . $class_name . '.class.php';
			if ($this->sneakClass($class_name, $engine_include_path)) {
				$query_include_path = $absolute_path . $path . '/models/' . $class_name . '.query.php';
				$this->sneakClass($class_name, $query_include_path);
				return true;
			}
			$engine_include_path = $absolute_path . $path . 'default.class.php';
			if ($this->sneakClass($class_name, $engine_include_path)) return true;
		}

		// Try the plugin files - if a plugin is currently running
		if (!empty($configuration['m'])) {
			$navigation = $this->PHPDS_navigation();
			if (!empty($navigation->navigation[$configuration['m']]['plugin'])) {

				// Check if file exists in active plugins folder.
				$plugin_include_class = $absolute_path . 'plugins/' . $navigation->navigation[$configuration['m']]['plugin'] . '/includes/' . $class_name . '.class.php';
				if ($this->sneakClass($class_name, $plugin_include_class)) {
					$plugin_include_query = $absolute_path . 'plugins/' . $navigation->navigation[$configuration['m']]['plugin'] . '/models/' . $class_name . '.query.php';
					$this->sneakClass($class_name, $plugin_include_query);
					return true;
				}

				$plugin_include_class = $absolute_path . 'plugins/' . $navigation->navigation[$configuration['m']]['plugin'] . '/includes/default.class.php';
				if ($this->sneakClass($class_name, $plugin_include_class)) return true;
			}
		}

		// Check PHPDS's own include folder
		$phpdev_include_class = $absolute_path . 'plugins/PHPDevShell/includes/' . $class_name . '.class.php';
		if ($this->sneakClass($class_name, $phpdev_include_class)) return true;

		// Oh this is becoming a problem, perhaps we can locate the file here.
		if (!empty($configuration['plugin_alt'])) {
			// Ok last chance, is it in here?
			$plugin_include_class = $absolute_path . $configuration['plugin_alt'] . 'includes/' . $class_name . '.class.php';
			if ($this->sneakClass($class_name, $plugin_include_class)) return true;
		}

		return false;
	}
}

/**
 * This is a base class for PHPDS subsystems
 *
 * It allows dependency injection and dependency fetching; also mimics multiple inheritance
 *
 * @date 20090427
 * @version	1.0.2 (greg)
 * @author greg
 *
 */
class PHPDS_dependant
{
	/**
	 * The object this object depend on. Ultimately up the chain it should be the main PHPSD instance
	 * @var PHPDS_dependant or PHPDS
	 */
	protected $dependance;
	/**
	 * The private debug instance of this object (with domain selection)
	 * @var debugInstance
	 */
	private $debugInstance;
	/**
	 * Holds a dependent extended object to appear as parent.
	 * @var object or name of the field containing the object
	 */
	protected $parent;

	/**
	 * magic constructor
	 *
	 * parameter list is not explicit: we're expecting the LAST argument to be the dependence, other are fed to construct()
	 *
	 * @author greg
	 *
	 */
	public function __construct()
	{
		$args = func_get_args();
		$dep = array_pop($args);
		$this->PHPDS_dependance($dep);

		$success = call_user_func_array(array($this, 'construct'), $args);

		if ($success === false)
				throw new PHPDS_Exception('Error constructing an object');
	}

	/**
	 * Empty function called by the actual constructor; meant to be overriden
	 *
	 * Supposed to return false (exactly) in case of error, otherwise return the object itself
	 *
	 * @return boolean or object
	 */
	public function construct() // variable argument list
	{
		return true;
	}

	/**
	 * Inject the given dependency into $this, and/or returns the owner
	 *
	 * The default behavior is to try to get the top owner, ie the main PHPDS instance.
	 * However the given parmeter is supposed to be the object from where the new object is created.
	 * That means you can override this to "catch" the real owner at this time.
	 *
	 * @date 20110202 (v1.1.1) (greg) added a check to prevent breaking on $this->dependancy if it's not an object
	 *
	 * @version 1.1.1
	 * @author greg
	 * @date 20100426
	 * @param object $dependance, the "owner", can be either PHPDS or PHPDS_dependant
	 * @return owner of $this
	 */
	public function PHPDS_dependance($dependance = null)
	{
		if (!empty($dependance) && empty($this->dependance)) {
			$this->dependance = $dependance;
		}
		if (!is_object($this->dependance)) throw new Exception("Dependancy error: dependance is not an object.");
		return method_exists($this->dependance, 'PHPDS_dependance') ? $this->dependance->PHPDS_dependance() : $this->dependance;
	}

	/**
	 * Magic php function, called when trying to access a non-defined field
	 *
	 * When a method want a field which doesn't exist, we assume it's a data from the father
	 *
	 * @version	1.0.2
	 * @author	greg
	 * @date 20100805 (v1.0.2) (greg) removed direct access to fields to avoid giving public access to private/protected fields
	 * @date		20100426		(v1.0.1) (greg) change to use PHPDS_dependance() to get the proper top owner
	 * @param $name	string name of the field to fetch
	 * @return mixed
	 */
	public function __get($name)
	{
		try {
			if (!empty($this->parent)) {
				if (isset($this->parent->{$name}) || property_exists($this->parent, $name))
						return $this->parent->{$name};
			}
			if ('debug' == $name) return $this->debugInstance();
			$top = $this->PHPDS_dependance();
			if (!method_exists($top, 'get')) {
				throw new Exception("Can't get any '$name' because dependancy is wrong.");
			}
			return $top->get($name);
		} catch (Exception $e) {
			throw new Exception("Can't get any '$name' because dependancy is wrong.");
		}
	}

	public function __set($name, $value)
	{
		try {
			if (!empty($this->parent)) {
				if (isset($this->parent->{$name}) || property_exists($this->parent->{$name}))
						$this->parent->{$name} = $value;
			}
		} catch (Exception $e) {
			throw new Exception("Can't set any '$name' because dependancy is wrong.");
		}
	}

	/**
	 * Magic php function used when a non-defined method is called. Here we mimics multi-inheritance by calling methods from "roots"
	 *
	 * @param string $name
	 * @param mixed $arguments
	 * @return mixed
	 */
	public function __call($name, $arguments)
	{
		$root = $this->parent;
		if (gettype($root) == 'string') $root = $this->{$root};
		if ((gettype($root) == 'object') && method_exists($root, $name)) {
			return call_user_func_array(array($root, $name), $arguments);
		}
		throw new Exception("No root have a \"$name()\" method.");
	}

	/**
	 * Create instance of PHPDS_debug.
	 *
	 * @date 20100412 (v1.0.1) (greg) deal with new sytle naming (starting with PHPDS_)
	 * @date 20100426 (v1.0.2) (greg) change to use PHPDS_dependance() to get the proper top owner
	 * @date 20110202 (v1.0.3) (greg) fixed a bug of using factoryWith(à instead of factory()
	 *
	 * @version 1.0.3
	 * @author greg
	 *
	 * @return object
	 */
	public function debugInstance($domain = null)
	{
		if (empty($this->debugInstance)) {
			if (empty($domain))
					$domain = preg_replace('/^PHPDS_/', '', get_class($this));
			$this->debugInstance = $this->factory('PHPDS_debug', $domain);
		}
		return $this->debugInstance;
	}

	/**
	 * Send info data to the debug subsystem (console, firebug, ...)
	 *
	 * The goal of this function is to be called all thourough the code to be able to track bugs.
	 *
	 * @date 20100928 (v1.1) (greg) this function send data with the DEBUG level
	 *
	 * @version 1.1
	 * @param string $data
	 */
	public function log($data)
	{
		if (is_a($this, 'debug')) $this->debug($data);
		else $this->debugInstance()->debug($data);
	}

	/**
	 * DEPRECATED: alias for log()
	 *
	 * @param $data
	 */
	public function _log($data)
	{
		return $this->log($data);
	}

	/**
	 * DEPRECATED: alias for _log()
	 *
	 * @param $data
	 */
	public function info($data)
	{
		$this->_log($data);
	}

	/**
	 * Create a new instance of the given class and link it as dependant (variable number of argument)
	 *
	 * @param string $classname, ... name of the class to instanciate, followed by class specific parameters
	 *
	 * @date 20100922 (v1.2) (greg) created factoryWith()
	 * @date 20100217 Split into factory() and PHPDS._factory() so the latter can be called with an array of arguments
	 * @date 20091125 Added the dependance link as the last parameter of the constructor
	 *
	 * @version 1.2
	 * @author greg
	 *
	 * @see PHPDS_debug
	 *
	 * @return  instance of $classname
	 */
	public function factory($classname) // actually more parameters can be given
	{
		$params = func_get_args();
		array_shift($params);

		return $this->factoryWith($classname, $params);
	}

	/**
	 * Create a new instance of the given class and link it as dependant (variable number of argument)
	 *
	 * @param string $classname name of the class to instanciate
	 * @param array $params all the parameters to feed the object's constructor (packed into an array)
	 *
	 * @version 1.0
	 * @author greg
	 *
	 * @return instance of $classname
	 */
	public function factoryWith($classname, $params)
	{
		$father = $this->PHPDS_dependance();
		// support for plugin aliasing
		if (!empty($father->PluginClasses[$classname]['class_name']))
				$classname = $father->PluginClasses[$classname]['class_name'];

		return $father->_factory($classname, $params, $this);
	}
}

/**
 * Turn core arrays into objects.
 *
 */
class PHPDS_array extends ArrayObject
{

	/**
	 * Magig set array.
	 *
	 * @param string $name
	 * @param string $val
	 */
	public function __set($name, $val)
	{
		$this[$name] = $val;
	}

	/**
	 * Magic get array.
	 *
	 * @param string $name
	 * @return string
	 */
	public function __get($name)
	{
		return (isset($this[$name]) ? $this[$name] : null);
	}
}